use Test::More;
use Test::Prereq;

SKIP: {
    unless (-d 'blib/lib') {

        # diag is visible w/ prove non-verbose
        diag "Test::Prereq must be run AFTER 'make'";
        skip "Test::Prereq must be run AFTER 'make'" , 1;
    }

    prereq_ok();
};

done_testing;
