#!/usr/bin/env perl

use 5.008; use strict;  use warnings;

BEGIN {
    if ($ENV{INSIDE_EMACS}) {
        chdir '..' until -d 't';
        use lib 'lib';
    }
}

use Test::More qw(no_plan);

subtest "sprintf() rounds x.5 to nearest even integer" => sub {
    is sprintf("%0.0f", 6.5), 6, "6.5 rounds down to 6";
    is sprintf("%0.0f", 7.5), 8, "7.5 rounds up to 8";
};

done_testing;
